      subroutine getorient(orient, mmax_views, view,
     +    angles_rad, angles_deg, center, imgmagfactor)

C     Unpacks the orient vector for a given view and returns the
C     following image parameters
C     
C     angles_rad(3) - (theta, phi, omega) in radians
C     angles_deg(3) - (theta, phi, omega) in degrees
C     center(2)     - (x, y)
C     imgmagfactor  - image magnification factor
C                                                               
C     Modification history:                                        
C     - Hong Lin, Version 1.0, Aug.18, 1998                    
C     - Hong Lin, Version 2.0, Oct. 7, 1999                    
C     - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002       
C     - R Sinkovits, cleanup and modernization 6/30/2006

      implicit none

      integer, intent(in) :: mmax_views, view
      real, intent(in)    :: orient( 0:6, mmax_views)

      real, intent(out) :: angles_rad(3), angles_deg(3), center(2), 
     +     imgmagfactor

      real deg_to_rad
      deg_to_rad = acos(-1.0)/180.0

      angles_deg(1) = orient(1, view)
      angles_deg(2) = orient(2, view)
      angles_deg(3) = orient(3, view)

      angles_rad(1) = angles_deg(1) * deg_to_rad
      angles_rad(2) = angles_deg(2) * deg_to_rad
      angles_rad(3) = angles_deg(3) * deg_to_rad

      center(1)     = orient(4, view)
      center(2)     = orient(5, view)

      imgmagfactor  = orient(6, view)

      return
      end subroutine getorient
