      subroutine intlz_params(hkl_max, num_hkl,
     +             min_xyz,max_xyz, nrffta_row, nrffta_col,
     +             nrffta_omega, nfc_2d_row, nfc_2d_col, 
     +             nrffts_1_row, 
     +             nrffts_1_col, nrffts_1_omega,
     +             nrffts_2_row, nrffts_2_col, nrffts_2_omega, 
     +             den_min_x, den_max_x, den_min_y, den_max_y, 
     +             den_min_z, den_max_z,
     +             confine_3D)
C                                                           INTLZ_PARAMS
C **********************************************************************
C *                                                               *
C * Subroutine: intlz_params                                      *
C * Author: Robert E. Lynch (rel@cs.purdue.edu)                   *
C * Date:   1997                                                  *
C * Used in: aamain -- The main routine.                          *
C * Function: Initialize parameters for 3D reconstruction.        *
C * Usage:                                                        *
C *   call intlz_params( hkl_max, num_hkl, min_xyz,               *
C *          max_xyz, nrffta_row, nrffta_col, nrffta_omega,       *
C *          nfc_2d_row, nfc_2d_col, nrffts_1_row,                *
C *          nrffts_1_col, nrffts_1_omega, nrffts_2_row,          *
C *          nrffts_2_col, nrffts_2_omega, den_min_x, den_max_x,  *
C *          den_min_y, den_max_y, den_min_z, den_max_z,          *
C *          confine_3D )                                *
C * Input:                                                        *
C *   confine_3D (Logical) -- Indicating if the size of 3D DFT is *
C *     restricted the same as that of pixel frame                *
C * Output:                                                       *
C *   hkl_max (Integer) -- The maximum index of HKL space.        *
C *   num_hkl (Integer) -- The number of indexes in each axe of   *
C *                        HKL space                              *
C *   min_xyz, max_xyz (Integer) -- The minimum and maximum index *
C *                                 of XYZ space                  *
C *   nrffta_row (Integer) -- The number of rows in work arrays   *
C *     into which the pixel values are moved, with zero fill.    *
C *   nrffta_col (Integer) -- The number of columns in work       *
C *     arrays into which the pixel values are moved, with zero   *
C *     fill.                                                     *
C *   nrffta_omega (Integer) -- Work space size for FFT analysis. *
C *   nfc_2d_row (Integer) -- The number of rows for complex      *
C *     array holding the Fourier coefficients of the pixel       *
C *     values, having the center as as specified in the PIF      *
C *     file header.                                              *
C *   nfc_2d_col (Integer) -- The number of columns for complex   *
C *     array holding the Fourier coefficients of the pixel       *
C *     values, having the center as as specified in the PIF file *
C *     header.                                                   *
C *   nrffts_1_row, nrffts_1_col (Integer) -- The number of rows  *
C *     and columns of array RFFTS_1                              *
C *   nrffts_1_omega (Integer) -- The size of array RFFTS_1_OMEGA *
C *   nrffts_2_row, nrffts_2_col (Integer) -- The number of rows  *
C *     and columns of array RFFTS_2                              *
C *   nrffts_2_omega (Integer) -- The size of array RFFTS_2_OMEGA *
C *   den_min_x, den_max_x (Integer) -- Limits of X indexes of    *
C *                                     the final density map     *
C *   den_min_y, den_max_y (Integer) -- Limits of Y indexes of    *
C *                                     the final density map     *
C *   den_min_z, den_max_z (Integer) -- Limits of Z indexes of    *
C *                                     the final density map     *
C *                                                               *
C * Algorithm:                                                    *
C *   See the comments on output arguments.                       *
C *                                                               *
C * Modification history:                                         *
C *  - Hong Lin, Version 0.6, Jul.6, 1998                      *
C *    Change the dependency among parameters.                 *
C *  - Hong Lin, Version 1.0, Aug.18, 1998                     *
C *  - Hong Lin, Version 2.0, Oct. 7, 1999                     *
C *   - Yongchang Ji, Final Version, Aug.2000 -- Jan. 2002             *

C
C Input variables are passed by COMMON in include files  info.inc ------

C npix_row       INTEGER*4; the number of max  rows and columns of
C         pixels in a frame of all the image files ; the values are read from the file header
C         and put into a COMMON block in  info.inc

C MAX_VIEW       INTEGER*4; the maximum number of pixel frames used in 
C         the computation (different nodes use different frames).  This
C         value is in the header of the PIF file.
 
C NUM_SYMM       INTEGER*4; the number of symmetry operators used.  This
C         value is part of the User's control input.
 
C   Output variables stored in COMMON in include file  allprogs.inc: ---

C PI             REAL*4;  3.141...
C TWOPI          REAL*4;  2*(3.141...)
C RAD_TO_DEG     REAL*4:  conversion fractor: radians to degrees.
C DEG_TO_RAD     REAL*4:  conversion fractor: degrees to radians.

C   Other output variables; argument of subroutine ---------------------

C HKL_MAX  INTEGER*4; the maximum values of H,K, or
C         L of the grid of Fourier coefficients used to synthesize the
C         output electron density.
 
C NUM_HKL    INTEGER*4; NUM_HKL .eq. 2*HKL_MAX + 1
 
C MAX_NUM_EQNS   INTEGER*4; MAX_VIEWS * NUM_SYMM = maximum number of
C         pixel frames times number of symmetry operators.  When
C         divided by the number of nodes, this is (one third) the
C         maximum number of equations solved by a node.
 
C NRFFTA_ROW, NRFFTA_COL  INTEGER*4; the number of rows and columns in
C         work arrays into which the pixel values are moved, with zero
C         fill. (The FFT analysis is carried out in the arrays RFFT and
C         RFFT_TRNS; the results are transfered to the complex array
C         FC_2D, and then the origin is translated.)
 
C NRFFTA_OMEGA   INTEGER*4; work space size for FFT analysis.
 
C NFC_2D_ROW, NFC_2D_COL  INTEGER*4; array parameters for complex array
C         holding the Fourier coefficients of the pixel values, having
C         the center as as specified in the PIF file header.
 
C ZERO_FILL      REAL*4; multiple of npix_row to get size NRFFTA_ROW;
C         ZERO_FILL is the ratio of side lengths of pixel frame and
C         array RFFTA in which FFT analysis take place (with zero fill)
 
C XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
C Following has to be corrected:

C ZERO_FILL      INTEGER*4; value set and used only in this subroutine.
C         The pixels in the square NPIX_ROW-by-NPIX_COL pixel frame are
C         put into an array ZERO_FILL-times-as-large, with zero fill.
C           The transformed values in frequency space are in a square
C         with side length = NPIX_ROW/(length of side of pixel frame).
C         This is independent of the value of ZERO_FILL.
C           The spacing of the discrete freqencies depends on the value
C         of ZERO_FILL = (length of edge of filled array)/NPIX_ROW.
C           The spacing of coefficents in frequency space is equal to
C         ZERO_FILL/NPIX_ROW.  Thus, ZERO_FILL .eq. 2 results in half
C         the spacing than when ZERO_FILL .eq. 1.  Use of ZERO_FILL .gt.
C         1 results in interpolation of the transform coefficients in
C         frequency space --- the larger ZERO_FILL, the finer the
C         spacing.
 
C ----------------------------------------------------------------------

      implicit none
      include '../include/allprogs.inc'
      include '../include/info.inc'
      
      integer, intent(out) :: hkl_max, num_hkl, 
     $     min_xyz, max_xyz, 
     $     nrffta_row, nrffta_col, nrffta_omega, nfc_2d_row, 
     $     nfc_2d_col, nrffts_1_row, nrffts_1_col, 
     $     nrffts_1_omega, nrffts_2_row, nrffts_2_col, 
     $     nrffts_2_omega, den_min_x, den_max_x, den_min_y, 
     $     den_max_y, den_min_z, den_max_z
      
      logical, intent(in) :: confine_3D

C     Local
      integer itemp
      character*12 subname
      data subname/ 'INTLZ_PARAMS'/
      logical debug
      data    debug / .false.  /

C     ----------------------------------------------------------------------
C     Set values of PI, TWOPI, RAD_TO_DEG, DEG_TO_RAD in a COMMON in
C     include file  allprogs.inc
C     
      pi         = 4.0 * atan(1.)
      twopi      = 2.0 * pi
      rad_to_deg = 180./ pi
      deg_to_rad = pi  / 180.
      
C     Replace the zero-fill factor:
      
      if( zero_fill < 1.0 ) then
         call error_stop(progname,subname,' ZERO_FILL must be >= 1')
      endif
      
      if(zero_fill < mapmagfactor .or. zero_fill < 1/mapmagfactor) then
         zero_fill = max(mapmagfactor, 1/mapmagfactor)
         print *, 'Adjust the Zero_Fill according to the mapmagfactor'
      endif
      
      itemp = nint( zero_fill * real( npix_row ) )
      call TRANS_LENGTH_ESSL( itemp, nrffta_row )
      zero_fill = real( nrffta_row ) / real( npix_row )
      
C     Make sure that arrays have an even number of rows
      nrffta_row = nrffta_row + mod(nrffta_row,2)
      nfc_2d_row = nrffta_row/2

      if(confine_3d) then
         itemp=nint(max(mapmagfactor,1/mapmagfactor)*real(npix_row))
         call TRANS_LENGTH_ESSL( itemp, num_hkl )
         hkl_max = num_hkl/ 2
      else
         hkl_max = NFC_2D_ROW
      endif

ccc      if (.not.confine_3D) then
C     
C     Set size of complex Fourier coefficients array generated by
C     FFT analysis --- the array is declared
C     COMPLEX FC_2D( 0:nfc_2d_row, -nfc_2d_col:nfc_2d_col )
C     with NFC_2D_ROW and NFC_2D_COL half the size of NRFFT_ROW
C     
ccc         nfc_2d_row   = nrffta_row/2
C     
C     Find the the limits on the spatial domain of the output
C     electron density array.
C     
C     The array of Fourier coefficients of the pixel values is
C     square and has 2*NFC_2D_COL+1 columns.  The plane of this
C     array has an has an arbitrary orientation because the plane
C     of pixels has an arbitrary orientation.  Thus the array of
C     Fouier coefficients has center the origin and is in a sphere
C     having radius sqrt( 2 * NFC_2D_COL**2 ).  Interpolation is
C     used in MATRIXENTRIES to estimate the corresponding Fourier
C     coefficients on a cubic grid with size 2*HKL_MAX + 1
C     We take HKL_MAX to be the smallest integer larger than the
C     radius of the sphere, sqrt( 2 * NFC_2D_COL )
C     (yji) change to  equal, I don't know why Bob use smaller size
ccc         hkl_max = NFC_2D_ROW
ccc      else
ccc         nfc_2d_row = nrffta_row/2
C     
C     (yji) confined 3D synthesis, but the 3D size num_hkl must suitable
C     to mapmagfactor, if no mapmagfactor, num_hkl = npix_row +1
C     
ccc         itemp=nint(max(mapmagfactor,1/mapmagfactor)*real(npix_row))
ccc         call TRANS_LENGTH_ESSL( itemp, num_hkl )
ccc         hkl_max    = num_hkl/ 2
ccc      endif
      
C     (yji) notice num_hkl = nrffta_row + 1 if no confined_3D
      
      num_hkl = 2*hkl_max + 1 
      if (iam == 0) print *, "num_hkl=",num_hkl
      nfc_2d_col   = nfc_2d_row
      
C     (yji) notice max_x - min_x + 1 = num_hkl - 1
      
      max_x = hkl_max
      min_x   = -max_x + 1
      min_y   = min_x
      max_y   = max_x
      min_z   = min_x
      max_z   = max_x
      min_xyz = min( min_x, min_y, min_z )
      max_xyz = max( max_x, max_y, max_z )
      
C     (yji) we apply different output density map size
      
      if (num_den_pix == 0) then 
         den_min_x = - npix_row * mapmagfactor / 2
      else 
         den_min_x = - num_den_pix / 2
      endif
      
      den_max_x = - den_min_x
      den_min_y = den_min_x
      den_max_y = den_max_x
      den_min_z = den_min_x
      den_max_z = den_max_x
      
C     Make the arrays square
C     Set length of work spaces used in the FFT analysis
      nrffta_col = nrffta_row
      nrffta_omega = nrffta_col + 15
      
C     Set the array square and FFT work space length  for 2D Fourier synthesis
      nrffts_1_row   = 2 * hkl_max
      nrffts_1_col   = 2 * hkl_max
      nrffts_1_omega = 2 * hkl_max + 15
      
C     Set the array square and FFT work space length  for 1D Fourier synthesis
      nrffts_2_row   = 2 * hkl_max
      nrffts_2_col   = 2 * hkl_max
      nrffts_2_omega = 2 * hkl_max + 15
      
      if( debug ) then
         print *, ' '
         print 8000, ' Debug:  INTLZ_PARAMS.f,  IAM = ', iam
 8000    format(a32,i10)
         print 8000, ' hkl_max                      = ', hkl_max      
         print 8000, ' num_hkl                      = ', num_hkl
         print 8000, ' min_xyz                      = ', min_xyz
         print 8000, ' max_xyz                      = ', max_xyz
         print 8000, ' npix_row                     = ', npix_row
         print 8000, ' hkl_max                      = ', hkl_max
         print 8000, ' nrffta_row                   = ', nrffta_row
         print 8000, ' nrffta_col                   = ', nrffta_col
         print 8000, ' nrffta_omega                 = ', nrffta_omega
         print 8000, ' nfc_2d_row                   = ', nfc_2d_row
         print 8000, ' nfc_2d_col                   = ', nfc_2d_col
         print 8000, ' nrffts_1_row                 = ', nrffts_1_row  
         print 8000, ' nrffts_1_col                 = ', nrffts_1_col  
         print 8000, ' nrffts_1_omega               = ', nrffts_1_omega
         print 8000, ' nrffts_2_row                 = ', nrffts_2_row  
         print 8000, ' nrffts_2_col                 = ', nrffts_2_col  
         print 8000, ' nrffts_2_omega               = ', nrffts_2_omega
         print 8000, ' den_min_x                    = ', den_min_x
         print 8000, ' den_max_x                    = ', den_max_x
         print 8000, ' den_min_y                    = ', den_min_y
         print 8000, ' den_max_y                    = ', den_max_y
         print 8000, ' den_min_z                    = ', den_min_z
         print 8000, ' den_max_z                    = ', den_max_z
         print 8010, ' zero_fill                    = ', zero_fill
         print 8020, ' confine_3D                   = ', confine_3D
 8010    format(a32,f16.5)
 8020    format(a32,L10)
      endif
      
      return
      end subroutine intlz_params
