      subroutine cmpt_intrps(hkl_max, interpolants,
     $     interpolants_size, t_analy, t_fftanaly, max_iq_slab,
     $     mem_cmpt, total_views , ctf_val, abs_ctf, algorithm)

      implicit none
      
      include 'mpif.h'
      include '../include/allprogs.inc'
      include '../include/info.inc'
      include '../include/com_01.inc'
      
C     Subroutine arguments
      integer, intent(in) :: hkl_max, max_iq_slab
      integer, intent(in) :: interpolants_size, algorithm
      real, intent(in) :: ctf_val(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      real, intent(in) :: abs_ctf(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col)
      integer, intent(inout) :: total_views
      double precision, intent(out) :: mem_cmpt, t_analy, t_fftanaly

#ifdef SPREC
      real, intent(out) :: interpolants(interpolants_size)
#else
      double precision, intent(out) :: interpolants(interpolants_size)
#endif
      
      
C     Local variables
      integer view, symm, i, j, max_node_eq, max_count, datalen
      integer start_view, end_view, num_views, alloc_stat
      integer*8 clock1, clock2, clockmax, clockrate, ticks
      
      real du, dv, num_annulus, omega_temp_1, omega_temp,
     +     angles_rad(3), angles_deg(3), center(2), imgmagfactor,
     +     theta_ref, phi_ref, x_ref, y_ref, 
     +     rot_mat(3,3), 
     +     ruv_mat_1(3,2,3), ruv_mat_2(3,2,3),
     +     ruv_sup_1(3,3), ruv_sup_2(3,3),
     +     theta_symm(60), phi_symm(60), omega_symm(60), 
     +     ico_x(60), ico_y(60), ico_z(60)

C     Allocatable arrays
      real, allocatable ::  orient(:,:), rffta_omega(:)
      real, allocatable :: pixels_array(:,:,:) 
      real, allocatable ::  hkl_dft(:)
      complex, allocatable :: fc_2d(:,:)
      
      double precision :: mem_orient = 0.0D0,
     +     mem_rffta_omega = 0.0D0,
     +     mem_fc_2d = 0.0D0,
     +     mem_hkl_dft  = 0.0D0, mem_interp = 0.0D0,
     +     mem_interpmax = 0.0D0, mem_pixels_array = 0.0D0
      
      character(len=12) :: subname = 'cmpt_intrps'
      
      call system_clock(count_max=clockmax, count_rate=clockrate)
      
      if (algorithm == 1) then
         max_count = 0
      else if (algorithm == 3) then
         max_count = ((hkl_max+1)**2) * 2 * 3
      endif
      
      allocate(orient(0:6,max_views),
     +     rffta_omega(nrffta_omega),
     +     fc_2d(0:nfc_2d_row, -nfc_2d_col:nfc_2d_col),
     +     pixels_array(npix_row, npix_col, view_perread),
     +     hkl_dft(max_count * 6),
     +     stat = alloc_stat)
      
      if(alloc_stat > 0) then
         call error_stop (progname, subname,
     +        'Error in allocating memory for orient')
      endif
      
      mem_orient       = dble(4*7*max_views)
      mem_rffta_omega  = dble(4 * nrffta_omega)
      mem_fc_2d        = dble(8 * (1 + nfc_2d_row)*(1 + 2*nfc_2d_col))
      mem_pixels_array = dble(4 * view_perread * npix_row * npix_col)
      mem_hkl_dft      = dble(4 * max_count * 6)
      
      mem_cmpt = mem_orient + mem_rffta_omega + 
     $           mem_fc_2d  + mem_pixels_array + mem_hkl_dft
      
C     Initialize FFT workspace rffta_omega
      call vrffti( nrffta_col, rffta_omega )

      call system_clock(clock1)      
      num_views = 0 
      
      if(iam == 0) then
         call readorient(orient, max_views, ort_file_name, num_views,
     +        npix_row, npix_col, oddeven)
         
         write(*,1001) iam, ort_file_name, num_views, max_views
 1001    format(/' Node = ',i2,':  Read orientation file: ',a/,
     +        12x,' Number of views     = ', i9, i9)
      endif
      
C     Broadcast orientation info from root to all other processes
      if(num_nodes > 1) then
         call mpi_bcast(num_views, 1, mpi_integer,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST num_views')
         
         call mpi_bcast(orient, 7*max_views, mpi_real,
     +        0, mpi_comm_world, mpi_err)
         if( mpi_err .ne. 0 ) call error_stop(progname,subname,
     +        'Error in calling MPI_BCAST orient')
      endif
      
      call system_clock(clock2)
      ticks = clock2-clock1
      if(ticks < 0) then
         ticks = clockmax + ticks
      endif
      t_analy = t_analy + dble(ticks)/dble(clockrate)
      
      total_views = total_views + num_views
      
C     calculate the radius in FFT space for 3D DFT
      num_annulus = real(nrffta_row)*PIXSIZ_OUT/res_max 
      if (num_annulus >= nrffta_row/2.0) then
         call error_stop(progname,subname,
     +        'Resolution is too high, should less than 2*PIXEL_SIZE')
      endif
      
C     we seperate all the views by view_perread, each time we read at
C     most view_perread * num_nodes pixels and deal with them
      
      
      do j = 0, (num_views-1)/(view_perread*num_nodes)
         call system_clock(clock1)
         
         start_view = 1 + j * view_perread*num_nodes
         end_view   = start_view + view_perread*num_nodes - 1
         if (end_view > num_views) end_view = num_views
         
C     the pix_file_name has been opened in aamain
C     Read the current end_view-start_view+1 pixel frames.
         
         call set_indices(num_nodes, start_view, end_view,
     +        frst_lst_view, max_node_eq )

C     processor 0 read pixels_array, then send to processor i
         do i = num_nodes - 1 , 0 , -1
            if (frst_lst_view(2, i) >= frst_lst_view(1, i)) then
               call read_files(pixels_array, frst_lst_view(1,i),
     +              frst_lst_view(2,i), i, orient)
            endif
         enddo
         
         call system_clock(clock2)
         ticks = clock2-clock1
         if(ticks < 0) then
            ticks = clockmax + ticks
         endif
         t_analy = t_analy + dble(ticks)/dble(clockrate)
         
         start_view = frst_lst_view(1,iam)
         end_view   = frst_lst_view(2,iam)
         datalen    = 0
        
         do view = start_view, end_view

C     Get the orientation angles and center of next pixel frame
C     Also changes OMEGA from the range (0,360) deg to (-90,270) deg.
C     Because the orient used by BIO not consistent with defination 
            
            orient(3,view) =  360 - orient(3,view) - 90
            call getorient( orient, max_views, view,
     +           angles_rad, angles_deg, center, imgmagfactor)
            
C     adjust the imgmagfactor according to its own pixel_size
            imgmagfactor = imgmagfactor * PIXSIZ_OUT / angstrom_per_pix
            
C     Calculate set of equivalent orientations based on application of
C     symmetry operations. Choices include icoshedral, eightfold,
C     dihedral, and asymmetric
            
            if(symm_code == 532) then

               call ico_vector(orient(1,view), orient(2,view), ico_x,
     $              ico_y, ico_z)

               do i = 1, 60
                  theta_symm(i) = acos(ico_z(i)) * rad_to_deg
                  if (ico_x(i) == 0 .and. ico_y(i) == 0) then
                     phi_symm(i) = orient(2, view)
                  else
                     phi_symm(i) = atan2(ico_y(i),ico_x(i)) * rad_to_deg
                  endif
                  omega_symm(i) = 0.
               enddo

               theta_ref = 90.0
               if(abs(theta_ref-theta_symm(1)) <= 1.0 ) then
                  theta_ref = 85.0
               endif

               phi_ref = 0.0
               if(abs(phi_ref-phi_symm(1)) <= 5.0 ) then
                  phi_ref = 20.0
               endif

               call ico_vector(theta_ref, phi_ref, ico_x,
     $              ico_y, ico_z)
               
               do i = 1, num_symm
                  angles_rad(1) = theta_symm(i) * deg_to_rad
                  angles_rad(2) = phi_symm(i)   * deg_to_rad
                  angles_rad(3) = omega_symm(i) * deg_to_rad
                  
C     Calculates the INVERSE rotation matrix
                  call setrotmat(angles_rad, rot_mat)
                  
                  x_ref = rot_mat(1,1) * ico_x(i) +
     $                    rot_mat(1,2) * ico_y(i) +
     $                    rot_mat(1,3) * ico_z(i)
                  y_ref = rot_mat(2,1) * ico_x(i) +
     $                    rot_mat(2,2) * ico_y(i) +
     $                    rot_mat(2,3) * ico_z(i)
                  if (abs(x_ref) < 0.001 .and. abs(y_ref) < 0.001) then
                      call error_stop(progname, subname,
     $                    'Error in calculating Omegas')
                  endif
                  
                  omega_temp = atan2(y_ref, x_ref)
                  if (i == 1) then
                     omega_temp_1 = omega_temp
                  endif
                  omega_symm(i) = mod(orient(3,view) + 
     $                 (omega_temp - omega_temp_1)*rad_to_deg + 720.,
     $                 360.)
                  
               enddo
               
            elseif (symm_code == 8) then

               call eight_symmetry(theta_symm, phi_symm, omega_symm,
     $              angles_deg)

            elseif (symm_code == 5 .or. symm_code == 52 .or.
     +              symm_code == 2 .or. symm_code == 222 .or.
     +              symm_code == 3 .or. symm_code == 32 .or.
     +              symm_code == 6 .or. symm_code == 622 .or.
     +              symm_code == 7 .or. symm_code == 72 .or.
     +              symm_code == 8 .or. symm_code == 822) then

               call dihedral_symmetry(theta_symm, phi_symm, omega_symm,
     $              angles_deg,symm_code, num_symm)

            elseif (symm_code == 1) then
               theta_symm(1) = angles_deg(1)
               phi_symm(1) = angles_deg(2)
               omega_symm(1) = angles_deg(3)
            else

               call error_stop(progname, subname,
     +              'Error: unsupported symmetry')

            endif

            
            call system_clock(clock1)
            
C     Compute Discrete Fourier Transform of pixel values:
            call fft_2dfft( pixels_array(1, 1, view-start_view+1), 
     +           npix_row, npix_col, center,
     +           nrffta_omega, rffta_omega, nrffta_row, 
     +           nrffta_col, nfc_2d_row, 
     +           nfc_2d_col, fc_2d )
            du = center(1) - real(int(center(1)))
            dv = center(2) - real(int(center(2)))
            call shift_fft(fc_2d, nfc_2d_row, nfc_2d_col, du, dv)
            
C     Apply CTF correction
            IF(CTFMODE > 0 .and. CTFMODE <= 4) then
               fc_2d = fc_2d * ctf_val ! Fortran 90 array syntax
            endif
            
            call system_clock(clock2)
            ticks = clock2-clock1
            if(ticks < 0) then
               ticks = clockmax + ticks
            endif
            t_fftanaly = t_fftanaly + dble(ticks)/dble(clockrate)

C     Accumalate contributions to interpolants arrays from all of the
C     symmetry related views of the particle image
            
            do symm = 1, num_symm
               angles_rad(1) = theta_symm(symm) * deg_to_rad
               angles_rad(2) = phi_symm(symm) * deg_to_rad
               angles_rad(3) = omega_symm(symm) * deg_to_rad
               
C     Set the rotation matrices which transform the plane containing
C     the projection of the particle to the (X,Y,Z) `standard space'
C     and get the inverse rot_mat
C     CALL SETROTMAT
C     **************
               call setrotmat( angles_rad, rot_mat) 
               
C     Set the matrix which is used to determine the intersection of
C     grid line in the standard 3-d space with the rotated place
C     containing the Fourier coefficients of the pixels
C     CALL ROTTRANS
C     *************
Cji            call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.5 )
Cji	    call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.5 )
               call rottrans_1( rot_mat, ruv_mat_1, ruv_sup_1, -0.999 )
               call rottrans_1( rot_mat, ruv_mat_2, ruv_sup_2,  0.999 )

               call matrixentries_slab( nfc_2d_row, nfc_2d_col, fc_2d,
     +              hkl_max, interpolants,
     +              hkl_dft, max_count, datalen,
     +              rot_mat, ruv_mat_1, ruv_mat_2, ruv_sup_1, ruv_sup_2,
     +              num_annulus, abs_ctf, imgmagfactor, mapmagfactor,
     +              algorithm)
               
               if (algorithm == 3) then
                  mem_interp = 0
                  call exch_intpol(hkl_dft, max_count, datalen,
     +                 interpolants, hkl_max, max_iq_slab, mem_interp)
                  mem_interpmax = max( mem_interpmax, mem_interp)
                  datalen = 0
               endif
               
            enddo
            
         enddo
                  
         if(algorithm == 3) then
            do view = 1, max_node_eq - (end_view - start_view + 1)
               do symm = 1, num_symm
                  mem_interp = 0
                  call exch_intpol(hkl_dft, max_count, datalen,
     +                 interpolants, hkl_max, max_iq_slab, mem_interp)
                  mem_interpmax = max( mem_interpmax, mem_interp)
                  datalen = 0
               enddo
            enddo
         endif
         
      enddo
      
      deallocate(orient, rffta_omega, fc_2d, pixels_array)
      
      return
      end subroutine cmpt_intrps
