      SUBROUTINE GET_XY (FFT1, FFT2, XCEN, YCEN, IDIM1, IDIM2,
     1     RES_MAX, FFT_ORIGX, FFT_ORIGY, flag,
     2     ipx, ipy, peakx1, peakx2, peaky1, peaky2, step1, step2)

C     ROUTINE FOR ESTIMATING THE ORIGINS OF ICOSAHEDRAL PARTICLES BY
C     CROSS-CORRELATION WITH CIRCULARLY-AVERAGED REFERENCE.
C     
C     NOTE: "RES_MIN" here = 1.0 (NO CHOICE ALLOWED)
C     
C     MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER
C     MODIFIED 17-DEC-1997 BY T. BAKER (Rid of EM.CMM)
C     Modified 22-aug-2006, R Sinkovits cleanup and rid of em_wrt

      IMPLICIT NONE

      INTEGER*4, INTENT(IN)  :: IDIM1, IDIM2
      REAL*4, INTENT(IN)     :: FFT1(2*IDIM1,IDIM2), FFT2(2*IDIM1,IDIM2)
      REAL*4, INTENT(IN)     :: RES_MAX, FFT_ORIGX, FFT_ORIGY

      INTEGER*4, INTENT(OUT) :: FLAG
      INTEGER*4, INTENT(OUT) :: IPX, IPY
      REAL*4, INTENT(OUT)    :: XCEN, YCEN
      REAL*4, INTENT(OUT)    :: PEAKX1, PEAKX2, PEAKY1, PEAKY2
      REAL*4, INTENT(OUT)    :: STEP1, STEP2

      REAL*4 RES_MIN
      INTEGER*4 alloc_stat
      REAL*4, allocatable :: CC_MAP(:,:)

      allocate(CC_MAP(IDIM1,IDIM2), stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         write(*,*)'Not enough memory for cc_map in get_xy()'
         stop
      endif

      RES_MIN = 1.0
      CALL CCF_FFT (FFT1, FFT2, CC_MAP, IDIM1, IDIM2, RES_MIN, RES_MAX)

      CALL PFTCC_PEAK (CC_MAP, IDIM1, IDIM2, XCEN, YCEN, FFT_ORIGX,
     $     FFT_ORIGY, flag, ipx, ipy, peakx1, peakx2, peaky1, peaky2,
     $     step1, step2)
      
      
      deallocate(CC_MAP)
      
      RETURN
      END SUBROUTINE GET_XY
      
